use std::path::Path;
use std::io::Write;

mod bin;
mod html;
mod js;

fn process_file<T: AsRef<Path>>(path: &T) -> Result<Vec<u8>, Box<dyn std::error::Error>> {
    let p = path.as_ref();
    match p.extension().map(|x| x.to_str()).unwrap_or(None) {
        Some("css") => html::process(p),
        Some("js") => js::process(p),
        Some("html") => html::process(p),
        Some("ico") => bin::process(p),
        Some(x) => Err(format!("Unknown extension: {}", x).into()),
        None => Err("No extension".into()),
    }
}

fn process_dir<T: AsRef<Path>, U: AsRef<Path>>(
    input: T,
    output: U,
) -> Result<(), Box<dyn std::error::Error>> {
    let input_dir = std::fs::read_dir(input)?;
    for entry in input_dir {
        let entry = entry?;
        let path = entry.path();
        let mut output_name = output.as_ref().to_owned();
        output_name.push(path.file_name().unwrap());
        if path.is_dir() {
            if !output_name.exists() {
                std::fs::create_dir(&output)?;
            }
            process_dir(path, &output_name)?;
        } else {
            let contents = process_file(&path)?;
            let mut f = std::fs::File::create(output_name)?;
            f.write_all(&contents)?;
        }
    }
    Ok(())
}

fn main() -> Result<(), Box<dyn std::error::Error>> {
    process_dir("input", "output")?;
    Ok(())
    // process(&["input/code.js", "input/index.html"])
}
